function condmean=condmean_kernel(y,x,nb_points,bandwidth)
% condmean=condmean_kernel(y,x,nb_points,bandwidth)
% 
% The function condmean_kernel estimates the conditional mean of y given x
% with the non-parametric method of the gaussian kernel.
%
% Input:
% - y is a column vector containing the explained values
% - x is a column vector containing the explaining factors
% - nb_points is the number of points where the density is estimated ( see
% the remark)
% - bandwith is an optional argument

%
% Output:
% - condmean is a matrix (2,nbpoint) where the first row contains the
% estimation points (x) and the second row contains the values of the
% conditional mean of y, for the corresponding x value.
%
% Remark:
% The density is estimated  between the min and the maximum of the sample
% of observations, for nb_points equidistant points.
%

% we count the length of the sample 
nbdata=length(x);

% The last input argument of the function gaussiankernel is optional. If no
% value is passed for bandwith, the program uses the so called "rule of
% thumb"

% nargin is a Matlab system variable that counts the number of input
% arguments that were introduced into the function. If nargin is identical
% to three, that means that no bandwidth was specified, and thus the
% bandwidth is the one given by the rule of thumb
if nargin==3 
    % no, then we apply the "rule of thumb"
    bandwidth=(nbdata^(-0.2))*(var(x)^0.5);
end
% creation of the grid of points
MaxObs=max(x);
MinObs=min(x);
step=(MaxObs-MinObs)/(nb_points-1);
points=[MinObs:step:MaxObs];

% We calculate all the points in one step by using the functions mean and
% the kronecker product.
% This is the most compact and efficient writing as
% it optimizes the use of the preprogrammed functions of Matlab.
% Matlab is an interpreted language (as opposed to compiled language), you
% should avoid the use of loops and you should "vectorize" the code at best
% to obtain a good performance.
% Check the help on different functions for more details.

% calculation of the joint density y times (top part of formula in A.IX-65):
tmp=mean(exp(-0.5*((kron(x,ones(1,nb_points))-kron(ones(nbdata,1),points))/bandwidth).^2).*kron(y,ones(1,nb_points)),1)/bandwidth/((2*pi)^0.5);
% calculation of the marginal density (bottom part of formula in A.IX-65):
tmp2=mean(exp(-0.5*((kron(x,ones(1,nb_points))-kron(ones(nbdata,1),points))/bandwidth).^2),1)/bandwidth/((2*pi)^0.5);

condmean=[points;tmp./tmp2]; %divide top by bottom to get formula in A.IX-65
